#!/usr/bin/env node
/*!
 * gitmars v4.0.0-alpha.21
 * 这是一个git工作流工具
 * (c) 2021-2022 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import sh from 'shelljs';
import chalk from 'chalk';
import getBranchesFromID from '@gitmars/core/lib/git/getBranchesFromID';
import getIsGitProject from '@gitmars/core/lib/git/getIsGitProject';
import getCurrentBranch from '@gitmars/core/lib/git/getCurrentBranch';
import getIsMergedTargetBranch from '@gitmars/core/lib/git/getIsMergedTargetBranch';
import getIsUpdatedInTime from '@gitmars/core/lib/git/getIsUpdatedInTime';
import getIsMergeAction from '@gitmars/core/lib/git/getIsMergeAction';
import getBehindLogs from '@gitmars/core/lib/git/getBehindLogs';
import { createArgs } from '@gitmars/core/lib/utils/command';
import { init, remove } from '@gitmars/core/lib/hook/index';
import getConfig from '@gitmars/core/lib/getConfig';
import lang from '#lib/common/local';
import hookConfig from '#lib/conf/hook';

const { t } = lang;
const { green, red } = chalk;
const { args, options } = hookConfig;
if (!getIsGitProject()) {
  sh.echo(red(t("The current directory is not a git project directory")));
  process.exit(1);
}
const config = getConfig();
program.name("gitm hook").usage(
  "[command] [args...] [--no-verify] [--lastet [lastet]] [--limit [limit]] [-t --type <type>] [--branch [branch]]"
).description("git hooks");
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(
  async (command, args2, opt) => {
    if (opt.noVerify) {
      process.exit(0);
      return;
    }
    if (command === "init") {
      init();
    } else if (command === "remove") {
      remove();
    } else {
      const types = opt.type ? opt.type.split(",") : [];
      const mainBranches = [
        config.master,
        config.develop,
        config.release,
        config.support,
        config.bugfix
      ];
      const current = getCurrentBranch();
      console.info(
        types,
        process.env,
        process.argv,
        getBranchesFromID("2080d17e")
      );
      if (current !== config.develop && mainBranches.includes(current) && types.includes("1")) {
        const [command2, branch] = process.env.GIT_REFLOG_ACTION ? process.env.GIT_REFLOG_ACTION.split(" ") : [];
        if (command2 === "merge") {
          const isMergedBranch = getIsMergedTargetBranch(
            branch,
            config.develop,
            true
          );
          if (!isMergedBranch) {
            console.info(
              red(
                t(
                  "Your branch was detected as not having merged {target}",
                  {
                    target: config.develop
                  }
                )
              )
            );
            process.exit(0);
          } else {
            console.info(
              green(
                t(
                  "{source} branch has merged {target} branch",
                  { source: branch, target: config.develop }
                )
              )
            );
          }
        }
      }
      if (mainBranches.includes(current) && types.includes("2")) {
        const [command2, branch] = process.env.GIT_REFLOG_ACTION ? process.env.GIT_REFLOG_ACTION.split(" ") : [];
        const branchPrefix = branch.split("/")[0];
        if (command2 === "merge") {
          const isUpdatedInTime = getIsUpdatedInTime({
            lastet: opt.lastet,
            branch
          });
          if (!isUpdatedInTime) {
            console.info(
              red(
                t(
                  "Detected that you have not update codes from {source} branch in 1 week",
                  { source: branchPrefix }
                )
              )
            );
            process.exit(0);
          } else {
            console.info(
              green(
                t(
                  "The {source} branch has updated from main branch",
                  {
                    source: branch
                  }
                )
              )
            );
          }
        }
      }
      if (mainBranches.includes(current) && types.includes("3")) {
        const isMergeAction = getIsMergeAction();
        if (!isMergeAction) {
          console.info(
            red(
              t(
                "Detects that you are modifying code directly in the master branch"
              )
            )
          );
          process.exit(0);
        } else {
          console.info(green(t("The last record was a merge record")));
        }
      }
      if (mainBranches.includes(current) && types.includes("4")) {
        const behindLogs = getBehindLogs();
        if (!behindLogs.length) {
          console.info(
            t(
              "Your local branch version is lagging behind the remote branch, please perform a pull first."
            )
          );
          process.exit(0);
        } else {
          console.info(
            green(
              t(
                "The local version is not behind the remote, you can push it directly"
              )
            )
          );
        }
      }
    }
    process.exit(0);
  }
);
program.parse(process.argv);
